'********************************************************
'*							*
'*		Propeller II ROM Booter			*
'*		Terasic DE2-115				*
'*							*
'*		Version 0.3				*
'*							*
'*		11/13/2013				*
'*							*
'********************************************************

CON

  rx_pin = 63
  tx_pin = 62
  spi_cs = 61
  spi_ck = 60
  spi_di = 59
  spi_do = 58

  base = $4000

DAT		orgh	0
		org
'
'
' Attempt to boot from serial
'
boot
		jnp	#rx_pin,#fail			'if rx_pin is low, skip serial

		setb	outb,#30			'make tx high output
		setb	dirb,#30

		call	#rx_bit				'measure low rx calibration pulses (host $F9 -> %1..010011111..)
		mov	threshold,delta			'and calculate threshold
		call	#rx_bit				'(any timeout results in flash boot)
		add	threshold,delta
		shr	threshold,#1

		mov	count,#250			'ready to receive/verify 250 lfsr bits
.in		call	#rx_bit				'receive bit ($FE/$FF) into c
		test	lfsr,#$01		wz	'get lfsr bit into nz
   if_c_eq_z	jmp	#fail				'if mismatch, boot from flash
		test	lfsr,#$B2		wc	'advance lfsr
		rcl	lfsr,#1
		djnz	count,#.in			'loop for next bit in

		mov	count,#250+8			'ready to transmit 250 lfsr bits + 8 version bits
.out		cmp	count,#8		wz	'if last 8 bits, set lfsr so that version will be output
	if_z	mov	lfsr,#$EF+1			'$EF->$55: Prop123-A9, $F0->$92: Prop123-A7, $F1->$A7: DE2-115, $F2->$xx: DE0-Nano
		test	lfsr,#$01		wz	'get lfsr/version bit into nz, z=1 on last iteration
		call	#wait_rx			'wait for rx low (convey incoming $F9 on rx_pin to $FE/$FF on tx_pin)
		clrb	outb,#tx_pin			'make tx low
		call	#wait_rx			'wait for rx high
		setbnz	outb,#tx_pin			'make tx lfsr/version bit
		call	#wait_rx			'wait for rx low
		setb	outb,#tx_pin			'make tx high
		call	#wait_rx			'wait for rx high
		test	lfsr,#$B2		wc	'advance lfsr
		rcl	lfsr,#1
		djnz	count,#.out			'loop for next bit out

		jmp	#load				'serial handshake done, attempt to load from serial (z=1)
'
'
' Receive bit (c) - compare incoming pulse to threshold
'
rx_bit		call	#wait_rx			'wait for rx low
		getct	x				'get time

		call	#wait_rx			'wait for rx high
		getct	delta
		sub	delta,x				'get time delta

		cmp	delta,threshold		wc	'compare time delta to threshold

		ret
'
'
' Wait for rx low/high - if timeout, attempt to boot from flash
'
wait_rx		getct	time				'ready timeout
		add	time,timeout

		setedg	edgv				'ready edge

		setq	time
		waitedg				wc	'wait for rx low/high with timeout

		xor	edgv,#%11 << 6			'toggle pos/neg edge

	if_nc	ret					'return if not timeout (fail follows)
'
'
' Fail, toggle p0
'
fail		mov	dira,pinm
		rep	#0,#0
		xor	outa,pinm
'
'
' Load from serial (z=1) or flash (z=0)
'
load		mov	ptra,basev			'load loader into $4000

		decod	count,#9			'ready to input $200 longs
.long		mov	bits,#32			'ready to input 32 data bits

.bit		call	#rx_bit				'input serial bit (serial mode)
		rcl	data,#1				'shift bit into long
		djnz	bits,#.bit			'loop, adequate time for next flash bit

		wrlong	data,ptra++			'store long in hub ram (ptra=base+$800 after)
		djnz	count,#.long			'loop for next long

		coginit	#0,basev			'relaunch cog0 with loader
'
'
' Constants
'
'timeout	long	20_000_000/1000*150		'150ms @20MHz (rcfast)
timeout		long	50_000_000/1000*150		'150ms @50MHz (clock_50)
edgv		long	%10_000000 | rx_pin
basev		long	base
lfsr		long	"P"
pinm		long	1
'
'
' Variables
'
count		res	1
bits		res	1
data		res	1
time		res	1
delta		res	1
threshold	res	1
x		res	1
y		res	1
